@@ -231,8 +231,8 @@
   },
   /* PREFIX_EVEX_0F3852 */
   {
-    { Bad_Opcode },
-    { "vdpbf16p%XS", { XM, Vex, EXx }, 0 },
+    { "vdpphp%XS",	{ XM, Vex, EXx }, 0 },
+    { "vdpbf16p%XS",	{ XM, Vex, EXx }, 0 },
     { VEX_W_TABLE (VEX_W_0F3852) },
     { "vp4dpws%XSd",	{ XM, Vex, Mxmm }, 0 },
   },
@@ -257,6 +257,13 @@
     { VEX_W_TABLE (EVEX_W_0F3872_P_2) },
     { "vcvtne2p%XS2bf16", { XM, Vex, EXx}, 0 },
   },
+  /* PREFIX_EVEX_0F3874 */
+  {
+    { "vcvtbiasp%XH2bf8",	{ XMxmmq, Vex, EXxh }, 0 },
+    { "vcvtnep%XH2bf8%XY",	{ XMxmmq, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vcvtne2p%XH2bf8",	{ XM, Vex, EXxh }, 0 },
+  },
   /* PREFIX_EVEX_0F389A */
   {
     { Bad_Opcode },
@@ -290,6 +297,7 @@
     { "vrndscalep%XH",  { XM, EXxh, EXxEVexS, Ib }, 0 },
     { Bad_Opcode },
     { "vrndscalep%XS",  { XM, EXx, EXxEVexS, Ib }, 0 },
+    { "vrndscalenep%XB",	{ XM, EXxh, Ib }, 0 },
   },
   /* PREFIX_EVEX_0F3A0A */
   {
@@ -302,6 +310,7 @@
     { "vgetmantp%XH",     { XM, EXxh, EXxEVexS, Ib }, 0 },
     { Bad_Opcode },
     { "vgetmantp%XW",	{ XM, EXx, EXxEVexS, Ib }, 0 },
+    { "vgetmantp%XB",	{ XM, EXxh, Ib }, 0 },
   },
   /* PREFIX_EVEX_0F3A27 */
   {
@@ -309,11 +318,18 @@
     { Bad_Opcode },
     { "vgetmants%XW",	{ XMScalar, VexScalar, EXdq, EXxEVexS, Ib }, 0 },
   },
+  /* PREFIX_EVEX_0F3A42_W_0 */
+  {
+    { Bad_Opcode },
+    { "%XEvmpsadbw",	{ XM, Vex, EXx, Ib }, 0 },
+    { "vdbpsadbw",	{ XM, Vex, EXx, Ib }, 0 },
+  },
   /* PREFIX_EVEX_0F3A56 */
   {
     { "vreducep%XH",      { XM, EXxh, EXxEVexS, Ib }, 0 },
     { Bad_Opcode },
     { "vreducep%XW",	{ XM, EXx, EXxEVexS, Ib }, 0 },
+    { "vreducenep%XB",	{ XM, EXxh, Ib }, 0 },
   },
   /* PREFIX_EVEX_0F3A57 */
   {
@@ -326,6 +342,7 @@
     { "vfpclassp%XH%XZ",  { MaskG, EXxh, Ib }, 0 },
     { Bad_Opcode },
     { "vfpclassp%XW%XZ",    { MaskG, EXx, Ib }, 0 },
+    { "vfpclassp%XB%XZ",	{ MaskG, EXxh, Ib }, 0 },
   },
   /* PREFIX_EVEX_0F3A67 */
   {
@@ -337,41 +354,15 @@
   {
     { "vcmpp%XH", { MaskG, Vex, EXxh, EXxEVexS, CMP }, 0 },
     { "vcmps%XH", { MaskG, VexScalar, EXw, EXxEVexS, CMP }, 0 },
-  },
-  /* PREFIX_EVEX_MAP4_D8 */
-  {
-    { "sha1nexte", { XM, EXxmm }, 0 },
-    { REG_TABLE (REG_0F38D8_PREFIX_1) },
-  },
-  /* PREFIX_EVEX_MAP4_DA */
-  {
-    { "sha1msg2", { XM, EXxmm }, 0 },
-    { "encodekey128", { Gd, Rd }, 0 },
-  },
-  /* PREFIX_EVEX_MAP4_DB */
-  {
-    { "sha256rnds2", { XM, EXxmm, XMM0 }, 0 },
-    { "encodekey256", { Gd, Rd }, 0 },
-  },
-  /* PREFIX_EVEX_MAP4_DC */
-  {
-    { "sha256msg1", { XM, EXxmm }, 0 },
-    { "aesenc128kl", { XM, M }, 0 },
-  },
-  /* PREFIX_EVEX_MAP4_DD */
-  {
-    { "sha256msg2", { XM, EXxmm }, 0 },
-    { "aesdec128kl", { XM, M }, 0 },
-  },
-  /* PREFIX_EVEX_MAP4_DE */
-  {
     { Bad_Opcode },
-    { "aesenc256kl", { XM, M }, 0 },
+    { "vcmpp%XB",	{ MaskG, Vex, EXxh, CMP }, 0 },
   },
-  /* PREFIX_EVEX_MAP4_DF */
+  /* PREFIX_EVEX_MAP4_4x */
   {
+    { "%CFcmov%CCS",	{ VexGv, { CFCMOV_Fixup, 0 }, { CFCMOV_Fixup, 1 } }, 0 },
     { Bad_Opcode },
-    { "aesdec256kl", { XM, M }, 0 },
+    { "%CFcmov%CCS",	{ VexGv, { CFCMOV_Fixup, 0 }, { CFCMOV_Fixup, 1 } }, 0 },
+    { "set%ZU%CC",	{ Eb }, 0 },
   },
   /* PREFIX_EVEX_MAP4_F0 */
   {
@@ -392,9 +383,9 @@
   /* PREFIX_EVEX_MAP4_F8 */
   {
     { Bad_Opcode },
-    { "enqcmds", { Gva, M },  0 },
+    { MOD_TABLE (MOD_EVEX_MAP4_F8_P_1) },
     { "movdir64b", { Gva, M }, 0 },
-    { "enqcmd", { Gva, M }, 0 },
+    { MOD_TABLE (MOD_EVEX_MAP4_F8_P_3) },
   },
   /* PREFIX_EVEX_MAP5_10 */
   {
@@ -406,12 +397,33 @@
     { Bad_Opcode },
     { "vmovs%XH", { EXwS, VexScalarR, XMScalar }, 0 },
   },
+  /* PREFIX_EVEX_MAP5_18 */
+  {
+    { "vcvtbiasp%XH2hf8",	{ XMxmmq, Vex, EXxh }, 0 },
+    { "vcvtnep%XH2hf8%XY",	{ XMxmmq, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vcvtne2p%XH2hf8",	{ XM, Vex, EXxh }, 0 },
+  },
+  /* PREFIX_EVEX_MAP5_1B */
+  {
+    { "vcvtbiasp%XH2hf8s",	{ XMxmmq, Vex, EXxh }, 0 },
+    { "vcvtnep%XH2hf8s%XY",	{ XMxmmq, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vcvtne2p%XH2hf8s",	{ XM, Vex, EXxh }, 0 },
+  },
   /* PREFIX_EVEX_MAP5_1D */
   {
     { "vcvtss2s%XH",      { XMScalar, VexScalar, EXd, EXxEVexR }, 0 },
     { Bad_Opcode },
     { "vcvtps2p%XHx%XY",  { XMxmmq, EXx, EXxEVexR }, 0 },
   },
+  /* PREFIX_EVEX_MAP5_1E */
+  {
+    { Bad_Opcode },
+    { Bad_Opcode },
+    { Bad_Opcode },
+    { "vcvthf82p%XH",	{ XM, EXxmmq }, 0 },
+  },
   /* PREFIX_EVEX_MAP5_2A */
   {
     { Bad_Opcode },
@@ -434,21 +446,26 @@
   /* PREFIX_EVEX_MAP5_2F */
   {
     { "vcomisY%XH",        { XMScalar, EXw, EXxEVexS }, 0 },
+    { Bad_Opcode },
+    { "vcoms%XB",	{ XMScalar, EXw, EXxEVexS }, 0 },
   },
   /* PREFIX_EVEX_MAP5_51 */
   {
     { "vsqrtp%XH",        { XM, EXxh, EXxEVexR }, 0 },
     { "vsqrts%XH",        { XMScalar, VexScalar, EXw, EXxEVexR }, 0 },
+    { "vsqrtnep%XB",	{ XM, EXxh }, 0 },
   },
   /* PREFIX_EVEX_MAP5_58 */
   {
     { "vaddp%XH", { XM, Vex, EXxh, EXxEVexR }, 0 },
     { "vadds%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, 0 },
+    { "vaddnep%XB",	{ XM, Vex, EXxh }, 0 },
   },
   /* PREFIX_EVEX_MAP5_59 */
   {
     { "vmulp%XH", { XM, Vex, EXxh, EXxEVexR }, 0 },
     { "vmuls%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, 0 },
+    { "vmulnep%XB", { XM, Vex, EXxh }, 0 },
   },
   /* PREFIX_EVEX_MAP5_5A */
   {
@@ -467,21 +484,32 @@
   {
     { "vsubp%XH", { XM, Vex, EXxh, EXxEVexR }, 0 },
     { "vsubs%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, 0 },
+    { "vsubnep%XB", { XM, Vex, EXxh }, 0 },
   },
   /* PREFIX_EVEX_MAP5_5D */
   {
     { "vminp%XH", { XM, Vex, EXxh, EXxEVexS }, 0 },
     { "vmins%XH", { XMScalar, VexScalar, EXw, EXxEVexS }, 0 },
+    { "vminp%XB", { XM, Vex, EXxh }, 0 },
   },
   /* PREFIX_EVEX_MAP5_5E */
   {
     { "vdivp%XH", { XM, Vex, EXxh, EXxEVexR }, 0 },
     { "vdivs%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, 0 },
+    { "vdivnep%XB", { XM, Vex, EXxh }, 0 },
   },
   /* PREFIX_EVEX_MAP5_5F */
   {
     { "vmaxp%XH", { XM, Vex, EXxh, EXxEVexS }, 0 },
     { "vmaxs%XH", { XMScalar, VexScalar, EXw, EXxEVexS }, 0 },
+    { "vmaxp%XB", { XM, Vex, EXxh }, 0 },
+  },
+  /* PREFIX_EVEX_MAP5_74 */
+  {
+    { "vcvtbiasp%XH2bf8s",	{ XMxmmq, Vex, EXxh }, 0 },
+    { "vcvtnep%XH2bf8s%XY",	{ XMxmmq, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vcvtne2p%XH2bf8s",	{ XM, Vex, EXxh }, 0 },
   },
   /* PREFIX_EVEX_MAP5_78 */
   {
@@ -527,6 +555,30 @@
     { Bad_Opcode },
     { "vcvtp%XH2psx",	{ XM, EXxmmqh, EXxEVexS }, 0 },
   },
+  /* PREFIX_EVEX_MAP6_2C */
+  {
+    { "vscalefnep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vscalefp%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_42 */
+  {
+    { "vgetexpp%XB",	{ XM, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vgetexpp%XH",	{ XM, EXxh, EXxEVexS }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_4C */
+  {
+    { "vrcpp%XB",	{ XM, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vrcpp%XH",	{ XM, EXxh }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_4E */
+  {
+    { "vrsqrtp%XB",	{ XM, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vrsqrtp%XH",	{ XM, EXxh }, 0 },
+  },
   /* PREFIX_EVEX_MAP6_56 */
   {
     { Bad_Opcode },
@@ -541,6 +593,78 @@
     { Bad_Opcode },
     { "vfcmaddcs%XH",     { { DistinctDest_Fixup, scalar_mode }, VexScalar, EXd, EXxEVexR }, 0 },
   },
+  /* PREFIX_EVEX_MAP6_98 */
+  {
+    { "vfmadd132nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfmadd132p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_9A */
+  {
+    { "vfmsub132nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfmsub132p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_9C */
+  {
+    { "vfnmadd132nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfnmadd132p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_9E */
+  {
+    { "vfnmsub132nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfnmsub132p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_A8 */
+  {
+    { "vfmadd213nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfmadd213p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_AA */
+  {
+    { "vfmsub213nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfmsub213p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_AC */
+  {
+    { "vfnmadd213nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfnmadd213p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_AE */
+  {
+    { "vfnmsub213nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfnmsub213p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_B8 */
+  {
+    { "vfmadd231nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfmadd231p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_BA */
+  {
+    { "vfmsub231nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfmsub231p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_BC */
+  {
+    { "vfnmadd231nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfnmadd231p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
+  /* PREFIX_EVEX_MAP6_BE */
+  {
+    { "vfnmsub231nep%XB",	{ XM, Vex, EXxh }, 0 },
+    { Bad_Opcode },
+    { "vfnmsub231p%XH",	{ XM, Vex, EXxh, EXxEVexR }, 0 },
+  },
   /* PREFIX_EVEX_MAP6_D6 */
   {
     { Bad_Opcode },
