@@ -408,7 +408,7 @@ static const struct dis386 evex_table[][256] = {
     { "vpblendm%DQ",	{ XM, Vex, EXx }, PREFIX_DATA },
     { "vblendmp%XW",	{ XM, Vex, EXx }, PREFIX_DATA },
     { "vpblendm%BW",	{ XM, Vex, EXx }, PREFIX_DATA },
-    { Bad_Opcode },
+    { "vcvt2p%XS2phx",	{ XM, Vex, EXx, EXxEVexR }, PREFIX_DATA },
     /* 68 */
     { PREFIX_TABLE (PREFIX_EVEX_0F3868) },
     { Bad_Opcode },
@@ -423,7 +423,7 @@ static const struct dis386 evex_table[][256] = {
     { "vpshldv%DQ",  { XM, Vex, EXx }, PREFIX_DATA },
     { PREFIX_TABLE (PREFIX_EVEX_0F3872) },
     { "vpshrdv%DQ",  { XM, Vex, EXx }, PREFIX_DATA },
-    { Bad_Opcode },
+    { PREFIX_TABLE (PREFIX_EVEX_0F3874) },
     { "vpermi2%BW",	{ XM, Vex, EXx }, PREFIX_DATA },
     { "vpermi2%DQ",	{ XM, Vex, EXx }, PREFIX_DATA },
     { "vpermi2p%XW",	{ XM, Vex, EXx }, PREFIX_DATA },
@@ -529,8 +529,8 @@ static const struct dis386 evex_table[][256] = {
     /* D0 */
     { Bad_Opcode },
     { Bad_Opcode },
-    { Bad_Opcode },
-    { Bad_Opcode },
+    { VEX_W_TABLE (VEX_W_0F38D2) },
+    { VEX_W_TABLE (VEX_W_0F38D3) },
     { Bad_Opcode },
     { Bad_Opcode },
     { Bad_Opcode },
@@ -938,10 +938,10 @@ static const struct dis386 evex_table[][256] = {
     { Bad_Opcode },
     { Bad_Opcode },
     /* 38 */
-    { "%NEccmp%SCB%DF",		{ Eb, Gb }, 0 },
-    { "%NEccmp%SCS%DF",		{ Ev, Gv }, PREFIX_NP_OR_DATA },
-    { "%NEccmp%SCB%SW%DF",	{ Gb, Eb }, 0 },
-    { "%NEccmp%SCS%SW%DF",	{ Gv, Ev }, PREFIX_NP_OR_DATA },
+    { "%NEccmp%SCB%DF",	{ Eb, Gb }, 0 },
+    { "%NEccmp%SCS%DF",	{ Ev, Gv }, PREFIX_NP_OR_DATA },
+    { "%NEccmp%SCB%DF",	{ Gb, EbS }, 0 },
+    { "%NEccmp%SCS%DF",	{ Gv, EvS }, PREFIX_NP_OR_DATA },
     { Bad_Opcode },
     { Bad_Opcode },
     { Bad_Opcode },
@@ -1193,13 +1193,13 @@ static const struct dis386 evex_table[][256] = {
     { Bad_Opcode },
     { Bad_Opcode },
     /* 18 */
+    { PREFIX_TABLE (PREFIX_EVEX_MAP5_18) },
     { Bad_Opcode },
     { Bad_Opcode },
-    { Bad_Opcode },
-    { Bad_Opcode },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP5_1B) },
     { Bad_Opcode },
     { PREFIX_TABLE (PREFIX_EVEX_MAP5_1D) },
-    { Bad_Opcode },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP5_1E) },
     { Bad_Opcode },
     /* 20 */
     { Bad_Opcode },
@@ -1296,7 +1296,7 @@ static const struct dis386 evex_table[][256] = {
     { Bad_Opcode },
     { Bad_Opcode },
     { Bad_Opcode },
-    { Bad_Opcode },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP5_74) },
     { Bad_Opcode },
     { Bad_Opcode },
     { Bad_Opcode },
@@ -1506,7 +1506,7 @@ static const struct dis386 evex_table[][256] = {
     { Bad_Opcode },
     { Bad_Opcode },
     { Bad_Opcode },
-    { "vscalefp%XH",      { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_2C) },
     { "vscalefs%XH",      { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
     { Bad_Opcode },
     { Bad_Opcode },
@@ -1531,7 +1531,7 @@ static const struct dis386 evex_table[][256] = {
     /* 40 */
     { Bad_Opcode },
     { Bad_Opcode },
-    { "vgetexpp%XH",      { XM, EXxh, EXxEVexS }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_42) },
     { "vgetexps%XH",      { XMScalar, VexScalar, EXw, EXxEVexS }, PREFIX_DATA },
     { Bad_Opcode },
     { Bad_Opcode },
@@ -1542,9 +1542,9 @@ static const struct dis386 evex_table[][256] = {
     { Bad_Opcode },
     { Bad_Opcode },
     { Bad_Opcode },
-    { "vrcpp%XH",	  { XM, EXxh }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_4C) },
     { "vrcps%XH",	  { XMScalar, VexScalar, EXw }, PREFIX_DATA },
-    { "vrsqrtp%XH",       { XM, EXxh }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_4E) },
     { "vrsqrts%XH",       { XMScalar, VexScalar, EXw }, PREFIX_DATA },
     /* 50 */
     { Bad_Opcode },
@@ -1628,13 +1628,13 @@ static const struct dis386 evex_table[][256] = {
     { "vfmaddsub132p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
     { "vfmsubadd132p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
     /* 98 */
-    { "vfmadd132p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_98) },
     { "vfmadd132s%XH",  { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfmsub132p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_9A) },
     { "vfmsub132s%XH",  { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfnmadd132p%XH", { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_9C) },
     { "vfnmadd132s%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfnmsub132p%XH", { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_9E) },
     { "vfnmsub132s%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
     /* A0 */
     { Bad_Opcode },
@@ -1646,13 +1646,13 @@ static const struct dis386 evex_table[][256] = {
     { "vfmaddsub213p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
     { "vfmsubadd213p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
     /* A8 */
-    { "vfmadd213p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_A8) },
     { "vfmadd213s%XH",  { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfmsub213p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_AA) },
     { "vfmsub213s%XH",  { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfnmadd213p%XH", { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_AC) },
     { "vfnmadd213s%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfnmsub213p%XH", { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_AE) },
     { "vfnmsub213s%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
     /* B0 */
     { Bad_Opcode },
@@ -1664,13 +1664,13 @@ static const struct dis386 evex_table[][256] = {
     { "vfmaddsub231p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
     { "vfmsubadd231p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
     /* B8 */
-    { "vfmadd231p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_B8) },
     { "vfmadd231s%XH",  { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfmsub231p%XH",  { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_BA) },
     { "vfmsub231s%XH",  { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfnmadd231p%XH", { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_BC) },
     { "vfnmadd231s%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
-    { "vfnmsub231p%XH", { XM, Vex, EXxh, EXxEVexR }, PREFIX_DATA },
+    { PREFIX_TABLE (PREFIX_EVEX_MAP6_BE) },
     { "vfnmsub231s%XH", { XMScalar, VexScalar, EXw, EXxEVexR }, PREFIX_DATA },
     /* C0 */
     { Bad_Opcode },
